signature MECHANICS =
sig

  (* we represent bodies as (mass, location, velocity) *)
  type body = Scalar.scalar * Plane.point * Plane.vec

  (* get the position of a body *)
  val position : body -> Plane.point

  (* for printing *)
  val bodyToString : body -> string

  (* universal gravitational constant for this universe *)
  val G : Scalar.scalar

  (* accOnPoint(p1,(m2,p2)) computes the
     acceleration on a body at p1 due to a body of mass m2 at p2 *)
  val accOnPoint : Plane.point * (Scalar.scalar * Plane.point) -> Plane.vec

  (*  accOn(body1, body2) computes the acceleration on body 1 due to body 2 *)
  val accOn : body * body -> Plane.vec

  (* stepBody (b, a, t) computes the new position and velocity of b
     after a t seconds given that a is the acceleration. *)
  val stepBody : body * Plane.vec * Scalar.scalar -> body
end

