(* scalars and operations on them *)
signature SPACE =
sig

  (* points and vectors and operations on them *)
  type point
  type vec

  (* add(v1, v2) evaluates to the sum of the vectors *)
  val add : vec * vec -> vec

  (* scale(v, c) evaluates to the scalar product of v with c *)
  val scale : vec * Scalar.scalar -> vec

  (* vecFromPoints(X,Y) is the vector from X to Y *)
  val vecFromPoints : point * point -> vec

  (* the center of the plane *)
  val origin : point

  (* Computes the distance between the argument points *)
  val distance : point * point -> Scalar.scalar

  (* Computes the magnitude of the given vector *)
  val mag : vec -> Scalar.scalar

  val vecToString : vec -> string
  val pointToString : point -> string

  (* equality of points *)
  val pointEqual : point * point -> bool

  (* Computes the location of the head of the vector
     if its tail is at the given point. *)
  val displace : point * vec -> point

  (* the head of a vector as a point in the plane *)
  val head : vec -> point

  (* produces the unit vector in the same direction as the given vector *)
  val unitVec : vec -> vec

  (* the zero vector *)
  val zero : vec

  (* given two points, gives the point at the middle of the line between them *)
  val midpoint : point * point -> point

  (* sums up the vectors given by mapping the function on the sequence *)
  val sum : ('a -> vec) * 'a Seq.seq -> vec

  (* do NOT use these two functions except for testing *)
  val cartcoord : point -> Scalar.scalar * Scalar.scalar
  val fromcoord : Scalar.scalar * Scalar.scalar -> point
end
